#include <Servo.h>

#include <ESP8266WiFi.h>
#include <ESP8266WiFiAP.h>
#include <ESP8266WiFiGeneric.h>
#include <ESP8266WiFiMulti.h>
#include <ESP8266WiFiScan.h>
#include <ESP8266WiFiSTA.h>
#include <ESP8266WiFiType.h>
#include <WiFiClient.h>
#include <WiFiClientSecure.h>
#include <WiFiServer.h>
#include <WiFiUdp.h>
#include <PololuMaestro.h>
#include <PubSubClient.h>
#include <SoftwareSerial.h>

/************ WIFI and MQTT INFORMATION (CHANGE THESE FOR YOUR SETUP) ******************/
#define wifi_ssid //enter your WIFI SSID
#define wifi_password  //enter your WIFI Password

#define mqtt_server  // Enter your MQTT server adderss or IP. I use my DuckDNS adddress (yourname.duckdns.org) in this field
#define mqtt_user //enter your MQTT username
#define mqtt_password //enter your password

SoftwareSerial maestroSerial(4, 5);

WiFiClient espClient;
PubSubClient client(espClient);

MicroMaestro maestro(maestroSerial);

int val;
int itsatrap = 0;


void setup() {
  maestroSerial.begin(9600);
  Serial.begin(115200);
  setup_wifi();
  maestro.setSpeed(0,1);

  client.setServer(mqtt_server, 1883); //CHANGE PORT HERE IF NEEDED
  client.setCallback(callback);

}


void setup_wifi() {

  delay(10);
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(wifi_ssid);

  WiFi.mode(WIFI_STA);
  WiFi.begin(wifi_ssid, wifi_password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }

  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
}

void callback(char* topic, byte* payload, unsigned int length) {
    char p[length + 1];
    memcpy(p, payload, length);
    p[length] = NULL;
    String message(p);
    String mytopic(topic);
    if (itsatrap == 0 && mytopic == "blind/bc/command" && message.equals("ON")){  
      maestro.setSpeed(0,2);
      maestro.setAcceleration(0, 10);
      delay(100);
      maestro.setTarget(0, 9000);
      client.publish("blind/bc/state", "ON");
      delay(1000);
      Serial.println("ON");
      maestro.setTarget(0,0);
      }
    else if (mytopic == "blind/bc/command" && message.equalsIgnoreCase("OFF")){
      maestro.setSpeed(0,2);
      maestro.setAcceleration(0, 10);
      delay(100);
      maestro.setTarget(0, 1600); 
      client.publish("blind/bc/state", "OFF");
      delay(1000);
      Serial.println("OFF");
      maestro.setTarget(0,0);
    }
    else if (mytopic == "blind/bc/level"){
      maestro.setSpeed(0,2);
      maestro.setAcceleration(0, 10);
      delay(100);
      val = message.toInt(); //converts command to integer to be used for positional arrangement
      val = map (val, 0, 99, 1600, 9000);
      maestro.setTarget(0,val);
      client.publish("blind/bc/state", "ON");
      Serial.println(val);
      delay(1000);
      maestro.setTarget(0,0);
      itsatrap = 1;
    }
    else{
        itsatrap = 0;
    }

}


void loop() {

  if (!client.connected()) {
    reconnect();
  }
  client.loop();
}

void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");
    // Attempt to connect
  if (client.connect("ESPBlindstl", mqtt_user, mqtt_password)) {
      Serial.println("connected");

      client.subscribe("blind/bc/command");
      client.subscribe("blind/bc/level");
      client.publish("blind/bc/state", "OFF");
    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}
